@@ -1,55 +1,17 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
- * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
- * reserved.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include "apr_arch_file_io.h"
@@ -58,6 +20,10 @@
 #include "apr_strings.h"
 #include "apr_errno.h"
 
+#ifdef HAVE_UTIME
+#include <utime.h>
+#endif
+
 static apr_filetype_e filetype_from_mode(mode_t mode)
 {
     apr_filetype_e type;
@@ -101,7 +67,7 @@ static apr_filetype_e filetype_from_mode(mode_t mode)
     return type;
 }
 
-static void fill_out_finfo(apr_finfo_t *finfo, struct stat *info,
+static void fill_out_finfo(apr_finfo_t *finfo, struct_stat *info,
                            apr_int32_t wanted)
 { 
     finfo->valid = APR_FINFO_MIN | APR_FINFO_IDENT | APR_FINFO_NLINK
@@ -125,11 +91,33 @@ static void fill_out_finfo(apr_finfo_t *finfo, struct stat *info,
      */
 }
 
+apr_status_t apr_file_info_get_locked(apr_finfo_t *finfo, apr_int32_t wanted,
+                                      apr_file_t *thefile)
+{
+    struct_stat info;
+
+    if (thefile->buffered) {
+        apr_status_t rv = apr_file_flush_locked(thefile);
+        if (rv != APR_SUCCESS)
+            return rv;
+    }
+
+    if (fstat(thefile->filedes, &info) == 0) {
+        finfo->pool = thefile->pool;
+        finfo->fname = thefile->fname;
+        fill_out_finfo(finfo, &info, wanted);
+        return (wanted & ~finfo->valid) ? APR_INCOMPLETE : APR_SUCCESS;
+    }
+    else {
+        return errno;
+    }
+}
+
 APR_DECLARE(apr_status_t) apr_file_info_get(apr_finfo_t *finfo, 
                                             apr_int32_t wanted,
                                             apr_file_t *thefile)
 {
-    struct stat info;
+    struct_stat info;
 
     if (thefile->buffered) {
         apr_status_t rv = apr_file_flush(thefile);
@@ -172,7 +160,7 @@ APR_DECLARE(apr_status_t) apr_file_attrs_set(const char *fname,
         return APR_SUCCESS;
 
     status = apr_stat(&finfo, fname, APR_FINFO_PROT, pool);
-    if (!APR_STATUS_IS_SUCCESS(status))
+    if (status)
         return status;
 
     /* ### TODO: should added bits be umask'd? */
@@ -222,7 +210,7 @@ APR_DECLARE(apr_status_t) apr_file_mtime_set(const char *fname,
     apr_finfo_t finfo;
 
     status = apr_stat(&finfo, fname, APR_FINFO_ATIME, pool);
-    if (!APR_STATUS_IS_SUCCESS(status)) {
+    if (status) {
         return status;
     }
 
@@ -262,7 +250,7 @@ APR_DECLARE(apr_status_t) apr_stat(apr_finfo_t *finfo,
                                    const char *fname, 
                                    apr_int32_t wanted, apr_pool_t *pool)
 {
-    struct stat info;
+    struct_stat info;
     int srv;
 
     if (wanted & APR_FINFO_LINK)
@@ -313,11 +301,4 @@ APR_DECLARE(apr_status_t) apr_stat(apr_finfo_t *finfo,
     }
 }
 
-/* Perhaps this becomes nothing but a macro?
- */
-APR_DECLARE(apr_status_t) apr_lstat(apr_finfo_t *finfo, const char *fname,
-                      apr_int32_t wanted, apr_pool_t *pool)
-{
-    return apr_stat(finfo, fname, wanted | APR_FINFO_LINK, pool);
-}
 
